@@ -11,10 +11,11 @@ import yaml
 from easydict import EasyDict
 
 from ding.utils import deep_merge_dicts
-from ding.envs import get_env_cls, get_env_manager_cls
+from ding.envs import get_env_cls, get_env_manager_cls, BaseEnvManager
 from ding.policy import get_policy_cls
-from ding.worker import BaseLearner, BaseSerialEvaluator, BaseSerialCommander, Coordinator, AdvancedReplayBuffer, \
-    get_parallel_commander_cls, get_parallel_collector_cls, get_buffer_cls, get_serial_collector_cls
+from ding.worker import BaseLearner, InteractionSerialEvaluator, BaseSerialCommander, Coordinator, \
+    AdvancedReplayBuffer, get_parallel_commander_cls, get_parallel_collector_cls, get_buffer_cls, \
+    get_serial_collector_cls, MetricSerialEvaluator, BattleInteractionSerialEvaluator
 from ding.reward_model import get_reward_model_cls
 from .utils import parallel_transform, parallel_transform_slurm, parallel_transform_k8s, save_config_formatted
 
@@ -89,6 +90,7 @@ class Config(object):
         with tempfile.TemporaryDirectory() as temp_config_dir:
             temp_config_file = tempfile.NamedTemporaryFile(dir=temp_config_dir, suffix=ext_name)
             temp_config_name = osp.basename(temp_config_file.name)
+            temp_config_file.close()
             shutil.copyfile(filename, temp_config_file.name)
 
             temp_module_name = osp.splitext(temp_config_name)[0]
@@ -98,7 +100,6 @@ class Config(object):
             cfg_dict = {k: v for k, v in module.__dict__.items() if not k.startswith('_')}
             del sys.modules[temp_module_name]
             sys.path.pop(0)
-            temp_config_file.close()
 
         cfg_text = filename + '\n'
         with open(filename, 'r') as f:
@@ -151,7 +152,7 @@ def save_config_py(config_: dict, path: str) -> NoReturn:
     config_string = str(config_)
     from yapf.yapflib.yapf_api import FormatCode
     config_string, _ = FormatCode(config_string)
-    config_string = config_string.replace('inf', 'float("inf")')
+    config_string = config_string.replace('inf,', 'float("inf"),')
     with open(path, "w") as f:
         f.write('exp_config = ' + config_string)
 
@@ -308,7 +309,7 @@ def compile_config(
         policy: type = None,
         learner: type = BaseLearner,
         collector: type = None,
-        evaluator: type = BaseSerialEvaluator,
+        evaluator: type = InteractionSerialEvaluator,
         buffer: type = AdvancedReplayBuffer,
         env: type = None,
         reward_model: type = None,
@@ -328,7 +329,7 @@ def compile_config(
         - policy (:obj:`type`): Policy class which is to be used in the following pipeline
         - learner (:obj:`type`): Input learner class, defaults to BaseLearner
         - collector (:obj:`type`): Input collector class, defaults to BaseSerialCollector
-        - evaluator (:obj:`type`): Input evaluator class, defaults to BaseSerialEvaluator
+        - evaluator (:obj:`type`): Input evaluator class, defaults to InteractionSerialEvaluator
         - buffer (:obj:`type`): Input buffer class, defaults to BufferManager
         - env (:obj:`type`): Environment class which is to be used in the following pipeline
         - reward_model (:obj:`type`): Reward model class which aims to offer various and valuable reward
@@ -346,7 +347,7 @@ def compile_config(
         if 'collector' not in create_cfg:
             create_cfg.collector = EasyDict(dict(type='sample'))
         if 'replay_buffer' not in create_cfg:
-            create_cfg.replay_buffer = EasyDict(dict(type='priority'))
+            create_cfg.replay_buffer = EasyDict(dict(type='advanced'))
         if env is None:
             env = get_env_cls(create_cfg.env)
         if env_manager is None:
@@ -379,6 +380,8 @@ def compile_config(
         else:
             env_config = EasyDict()  # env does not have default_config
         env_config = deep_merge_dicts(env_config_template, env_config)
+        if env_manager is None:
+            env_manager = BaseEnvManager  # for compatibility
         env_config.manager = deep_merge_dicts(env_manager.default_config(), env_config.manager)
         policy_config = policy.default_config()
         policy_config = deep_merge_dicts(policy_config_template, policy_config)
@@ -390,26 +393,32 @@ def compile_config(
         learner.default_config(),
         policy_config.learn.learner,
     )
-    policy_config.collect.collector = compile_collector_config(policy_config, cfg, collector)
+    if create_cfg is not None or collector is not None:
+        policy_config.collect.collector = compile_collector_config(policy_config, cfg, collector)
     policy_config.eval.evaluator = deep_merge_dicts(
         evaluator.default_config(),
         policy_config.eval.evaluator,
     )
-    policy_config.other.replay_buffer = compile_buffer_config(policy_config, cfg, buffer)
+    if create_cfg is not None or buffer is not None:
+        policy_config.other.replay_buffer = compile_buffer_config(policy_config, cfg, buffer)
     default_config = EasyDict({'env': env_config, 'policy': policy_config})
     if len(reward_model_config) > 0:
         default_config['reward_model'] = reward_model_config
     cfg = deep_merge_dicts(default_config, cfg)
     cfg.seed = seed
     # check important key in config
-    assert all([k in cfg.env for k in ['n_evaluator_episode', 'stop_value']]), cfg.env
-    cfg.policy.eval.evaluator.stop_value = cfg.env.stop_value
-    cfg.policy.eval.evaluator.n_episode = cfg.env.n_evaluator_episode
+    if evaluator in [InteractionSerialEvaluator, BattleInteractionSerialEvaluator]:  # env interaction evaluation
+        assert all([k in cfg.env for k in ['n_evaluator_episode', 'stop_value']]), cfg.env
+        cfg.policy.eval.evaluator.stop_value = cfg.env.stop_value
+        cfg.policy.eval.evaluator.n_episode = cfg.env.n_evaluator_episode
     if 'exp_name' not in cfg:
         cfg.exp_name = 'default_experiment'
     if save_cfg:
         if not os.path.exists(cfg.exp_name):
-            os.mkdir(cfg.exp_name)
+            try:
+                os.mkdir(cfg.exp_name)
+            except FileExistsError:
+                pass
         save_path = os.path.join(cfg.exp_name, save_path)
         save_config(cfg, save_path, save_formatted=True)
     return cfg
@@ -451,7 +460,7 @@ def compile_config_parallel(
     """
     # for compatibility
     if 'replay_buffer' not in create_cfg:
-        create_cfg.replay_buffer = EasyDict(dict(type='priority'))
+        create_cfg.replay_buffer = EasyDict(dict(type='advanced'))
     # env
     env = get_env_cls(create_cfg.env)
     if 'default_config' in dir(env):
