@@ -1,6 +1,5 @@
-from abc import ABC
 from types import MethodType
-from typing import Type, Union, Any, List, Callable, Iterable, Dict, Optional
+from typing import Union, Any, List, Callable, Dict, Optional
 from functools import partial, wraps
 from easydict import EasyDict
 import copy
@@ -8,13 +7,10 @@ import platform
 from collections import namedtuple
 import numbers
 import logging
-import torch
 import enum
 import time
 import traceback
-import signal
-from ding.torch_utils import to_tensor, to_ndarray, to_list
-from ding.utils import ENV_MANAGER_REGISTRY, import_module, deep_merge_dicts, one_time_warning
+from ding.utils import ENV_MANAGER_REGISTRY, import_module, one_time_warning
 from ding.envs.env.base_env import BaseEnvTimestep
 from ding.utils.time_helper import WatchDog
 
@@ -50,14 +46,14 @@ def retry_wrapper(func: Callable = None, max_retry: int = 10, waiting_time: floa
             except BaseException as e:
                 exceptions.append(e)
                 time.sleep(waiting_time)
-        e_info = ''.join(
-            [
-                'Retry {} failed from:\n {}\n'.format(i, ''.join(traceback.format_tb(e.__traceback__)) + str(e))
-                for i, e in enumerate(exceptions)
-            ]
+        logging.error("Function {} has exceeded max retries({})".format(func, max_retry))
+        runtime_error = RuntimeError(
+            "Function {} has exceeded max retries({}), and the latest exception is: {}".format(
+                func, max_retry, repr(exceptions[-1])
+            )
         )
-        func_exception = Exception("Function {} runtime error:\n{}".format(func, e_info))
-        raise RuntimeError("Function {} has exceeded max retries({})".format(func, max_retry)) from func_exception
+        runtime_error.__traceback__ = exceptions[-1].__traceback__
+        raise runtime_error
 
     return wrapper
 
@@ -156,7 +152,7 @@ class BaseEnvManager(object):
     def ready_obs(self) -> Dict[int, Any]:
         """
         Overview:
-            Get the next observations(in ``torch.Tensor`` type) and corresponding env id.
+            Get the next observations(in ``np.ndarray`` type) and corresponding env id.
         Return:
             A dictionary with observations and their environment IDs.
         Example:
@@ -322,8 +318,7 @@ class BaseEnvManager(object):
             return self._envs[env_id].step(act)
 
         try:
-            ret = step_fn()
-            return ret
+            return step_fn()
         except Exception as e:
             self._env_states[env_id] = EnvState.ERROR
             raise e
