@@ -70,9 +70,9 @@ class TestSubprocessEnvManager:
             env_manager.step([])
 
     @pytest.mark.unittest
-    def test_error(self, setup_async_manager_cfg):
-        env_fn = setup_async_manager_cfg.pop('env_fn')
-        env_manager = SyncSubprocessEnvManager(env_fn, setup_async_manager_cfg)
+    def test_error(self, setup_sync_manager_cfg):
+        env_fn = setup_sync_manager_cfg.pop('env_fn')
+        env_manager = SyncSubprocessEnvManager(env_fn, setup_sync_manager_cfg)
         # Test reset error
         with pytest.raises(AssertionError):
             env_manager.reset(reset_param={i: {'stat': 'stat_test'} for i in range(env_manager.env_num)})
@@ -120,7 +120,7 @@ class TestSubprocessEnvManager:
     @pytest.mark.tmp  # gitlab ci and local test pass, github always fail
     def test_block(self, setup_async_manager_cfg, setup_watchdog, setup_model_type):
         env_fn = setup_async_manager_cfg.pop('env_fn')
-        env_manager = SyncSubprocessEnvManager(env_fn, setup_async_manager_cfg)
+        env_manager = AsyncSubprocessEnvManager(env_fn, setup_async_manager_cfg)
         watchdog = setup_watchdog(60)
         model = setup_model_type()
         # Test reset timeout
