@@ -105,7 +105,7 @@ class C51DQN(nn.Module):
         obs_shape: Union[int, SequenceType],
         action_shape: Union[int, SequenceType],
         encoder_hidden_size_list: SequenceType = [128, 128, 64],
-        head_hidden_size: int = 64,
+        head_hidden_size: int = None,
         head_layer_num: int = 1,
         activation: Optional[nn.Module] = nn.ReLU(),
         norm_type: Optional[str] = None,
@@ -132,6 +132,8 @@ class C51DQN(nn.Module):
         super(C51DQN, self).__init__()
         # For compatibility: 1, (1, ), [4, 32, 32]
         obs_shape, action_shape = squeeze(obs_shape), squeeze(action_shape)
+        if head_hidden_size is None:
+            head_hidden_size = encoder_hidden_size_list[-1]
         # FC Encoder
         if isinstance(obs_shape, int) or len(obs_shape) == 1:
             self.encoder = FCEncoder(obs_shape, encoder_hidden_size_list, activation=activation, norm_type=norm_type)
@@ -585,7 +587,7 @@ class DRQN(nn.Module):
             - action_shape (:obj:`Union[int, SequenceType]`): Action's space.
             - encoder_hidden_size_list (:obj:`SequenceType`): Collection of ``hidden_size`` to pass to ``Encoder``
             - head_hidden_size (:obj:`Optional[int]`): The ``hidden_size`` to pass to ``Head``.
-            - lstm_type (:obj:`Optional[str]`): Version of lstm cell, now support ``['normal', 'pytorch']``
+            - lstm_type (:obj:`Optional[str]`): Version of rnn cell, now support ['normal', 'pytorch', 'hpc', 'gru']
             - activation (:obj:`Optional[nn.Module]`):
                 The type of activation function to use in ``MLP`` the after ``layer_fn``,
                 if ``None`` then default set to ``nn.ReLU()``
@@ -629,13 +631,20 @@ class DRQN(nn.Module):
                 head_hidden_size, action_shape, head_layer_num, activation=activation, norm_type=norm_type
             )
 
-    def forward(self, inputs: Dict, inference: bool = False) -> Dict:
+    def forward(
+            self, inputs: Dict, inference: bool = False, saved_hidden_state_timesteps: Optional[list] = None
+    ) -> Dict:
         r"""
         Overview:
             Use observation tensor to predict DRQN output.
             Parameter updates with DRQN's MLPs forward setup.
         Arguments:
             - inputs (:obj:`Dict`):
+            - inference: (:obj:'bool'): if inference is True, we unroll the one timestep transition,
+                if inference is False, we unroll the sequence transitions.
+            - saved_hidden_state_timesteps: (:obj:'Optional[list]'): when inference is False,
+                we unroll the sequence transitions, then we would save rnn hidden states at timesteps
+                that are listed in list saved_hidden_state_timesteps.
 
        ArgumentsKeys:
             - obs (:obj:`torch.Tensor`): Encoded observation
@@ -679,14 +688,25 @@ class DRQN(nn.Module):
             return x
         else:
             assert len(x.shape) in [3, 5], x.shape
-            x = parallel_wrapper(self.encoder)(x)
+            x = parallel_wrapper(self.encoder)(x)  # (T, B, N)
             lstm_embedding = []
+            # TODO(nyz) how to deal with hidden_size key-value
+            hidden_state_list = []
+            if saved_hidden_state_timesteps is not None:
+                saved_hidden_state = []
             for t in range(x.shape[0]):  # T timesteps
-                output, prev_state = self.rnn(x[t:t + 1], prev_state)
+                output, prev_state = self.rnn(x[t:t + 1], prev_state)  # output: (1,B, head_hidden_size)
+                if saved_hidden_state_timesteps is not None and t + 1 in saved_hidden_state_timesteps:
+                    saved_hidden_state.append(prev_state)
                 lstm_embedding.append(output)
-            x = torch.cat(lstm_embedding, 0)
-            x = parallel_wrapper(self.head)(x)
-            x['next_state'] = prev_state
+                hidden_state = list(zip(*prev_state))  # {list: 2{tuple: B{Tensor:(1, 1, head_hidden_size}}}
+                hidden_state_list.append(torch.cat(hidden_state[0], dim=1))
+            x = torch.cat(lstm_embedding, 0)  # (T, B, head_hidden_size)
+            x = parallel_wrapper(self.head)(x)  # (T, B, action_shape)
+            x['next_state'] = prev_state  # the last timestep state including h and c
+            x['hidden_state'] = torch.cat(hidden_state_list, dim=-3)  # the all hidden state h
+            if saved_hidden_state_timesteps is not None:
+                x['saved_hidden_state'] = saved_hidden_state  # the selected saved hidden states, including h and c
             return x
 
 
