@@ -5,23 +5,32 @@ from .qrdqn import QRDQNPolicy
 from .c51 import C51Policy
 from .rainbow import RainbowDQNPolicy
 from .ddpg import DDPGPolicy
+from .d4pg import D4PGPolicy
 from .td3 import TD3Policy
+from .td3_bc import TD3BCPolicy
 from .a2c import A2CPolicy
 from .ppo import PPOPolicy
 from .sac import SACPolicy
+from .cql import CQLPolicy, CQLDiscretePolicy
 from .impala import IMPALAPolicy
+from .ngu import NGUPolicy
 from .r2d2 import R2D2Policy
 from .ppg import PPGPolicy
 from .sqn import SQNPolicy
 
 from .qmix import QMIXPolicy
+from .wqmix import WQMIXPolicy
 from .coma import COMAPolicy
 from .collaq import CollaQPolicy
 from .atoc import ATOCPolicy
 from .acer import ACERPolicy
+from .qtran import QTRANPolicy
 
 from .il import ILPolicy
 
+from .r2d3 import R2D3Policy
+
 from .command_mode_policy_instance import *
 
 from .policy_factory import PolicyFactory
+from .pdqn import PDQNPolicy
