@@ -120,7 +120,6 @@ class ACERPolicy(Policy):
             self._model.critic.parameters(),
             lr=self._cfg.learn.learning_rate_critic,
         )
-
         self._target_model = copy.deepcopy(self._model)
         self._target_model = model_wrap(
             self._target_model,
@@ -230,6 +229,10 @@ class ACERPolicy(Policy):
             # Calculate retrace
             q_retraces = compute_q_retraces(q_values, v_pred, rewards, actions, weights, ratio, self._gamma)
 
+        # the terminal states' weights are 0. it needs to be shift to count valid state
+        weights_ext = torch.ones_like(weights)
+        weights_ext[1:] = weights[0:-1]
+        weights = weights_ext
         q_retraces = q_retraces[0:-1]  # shape T,B,1
         q_values = q_values[0:-1]  # shape T,B,env_action_shape
         v_pred = v_pred[0:-1]  # shape T,B,1
@@ -263,6 +266,10 @@ class ACERPolicy(Policy):
         self._optimizer_critic.step()
         self._target_model.update(self._learn_model.state_dict())
 
+        with torch.no_grad():
+            kl_div = avg_pi * ((avg_pi + EPS).log() - (target_pi + EPS).log())
+            kl_div = (kl_div.sum(-1) * weights).sum() / total_valid
+
         return {
             'cur_actor_lr': self._optimizer_actor.defaults['lr'],
             'cur_critic_lr': self._optimizer_critic.defaults['lr'],
@@ -271,6 +278,7 @@ class ACERPolicy(Policy):
             'policy_loss': total_actor_loss.item(),
             'critic_loss': critic_loss.item(),
             'entropy_loss': (entropy_loss.sum() / total_valid).item(),
+            'kl_div': kl_div.item()
         }
 
     def _reshape_data(
@@ -314,10 +322,7 @@ class ACERPolicy(Policy):
         rewards = data['reward']  # shape T,B
         weights_ = 1 - data['done']  # shape T,B
         weights = torch.ones_like(rewards)  # shape T,B
-        weights[1:, ...] = weights_[:-1, ...]
-        rewards = rewards * weights
-        # weights= weights_
-        # rewards = rewards  # shape T,B
+        weights = weights_
         return target_logit, behaviour_logit, avg_action_logit, actions, values, rewards, weights
 
     def _state_dict_learn(self) -> Dict[str, Any]:
@@ -329,6 +334,7 @@ class ACERPolicy(Policy):
         """
         return {
             'model': self._learn_model.state_dict(),
+            'target_model': self._target_model.state_dict(),
             'actor_optimizer': self._optimizer_actor.state_dict(),
             'critic_optimizer': self._optimizer_critic.state_dict(),
         }
@@ -345,6 +351,7 @@ class ACERPolicy(Policy):
             complicated operation.
         """
         self._learn_model.load_state_dict(state_dict['model'])
+        self._target_model.load_state_dict(state_dict['target_model'])
         self._optimizer_actor.load_state_dict(state_dict['actor_optimizer'])
         self._optimizer_critic.load_state_dict(state_dict['critic_optimizer'])
 
@@ -474,4 +481,4 @@ class ACERPolicy(Policy):
             The user can define and use customized network model but must obey the same interface definition indicated \
             by import_names path. For IMPALA, ``ding.model.interface.IMPALA``
         """
-        return ['actor_loss', 'bc_loss', 'policy_loss', 'critic_loss', 'entropy_loss']
+        return ['actor_loss', 'bc_loss', 'policy_loss', 'critic_loss', 'entropy_loss', 'kl_div']
