@@ -75,14 +75,16 @@ class CollaQPolicy(Policy):
             # (float) The weight of collaq MARA loss
             collaq_loss_weight=1.0,
             # (float)
-            clip_value=1.5,
+            clip_value=100,
+            # (bool) Whether to use double DQN mechanism(target q for surpassing over estimation)
+            double_q=False,
         ),
         collect=dict(
             # (int) Only one of [n_sample, n_episode] shoule be set
-            # n_sample=32 * 16,
+            n_episode=32,
             # (int) Cut trajectories into pieces with length "unroll_len", the length of timesteps
             # in each forward when training. In qmix, it is greater than 1 because there is RNN.
-            unroll_len=20,
+            unroll_len=10,
         ),
         eval=dict(),
         other=dict(
@@ -202,7 +204,13 @@ class CollaQPolicy(Policy):
         ret = self._learn_model.forward(inputs, single_step=False)
         total_q = ret['total_q']
         agent_colla_alone_q = ret['agent_colla_alone_q'].sum(-1).sum(-1)
-        next_inputs = {'obs': data['next_obs']}
+
+        if self._cfg.learn.double_q:
+            next_inputs = {'obs': data['next_obs']}
+            logit_detach = self._learn_model.forward(next_inputs, single_step=False)['logit'].clone().detach()
+            next_inputs = {'obs': data['next_obs'], 'action': logit_detach.argmax(dim=-1)}
+        else:
+            next_inputs = {'obs': data['next_obs']}
         with torch.no_grad():
             target_total_q = self._target_model.forward(next_inputs, single_step=False)['total_q']
 
@@ -251,6 +259,7 @@ class CollaQPolicy(Policy):
         """
         return {
             'model': self._learn_model.state_dict(),
+            'target_model': self._target_model.state_dict(),
             'optimizer': self._optimizer.state_dict(),
         }
 
@@ -266,6 +275,7 @@ class CollaQPolicy(Policy):
             complicated operation.
         """
         self._learn_model.load_state_dict(state_dict['model'])
+        self._target_model.load_state_dict(state_dict['target_model'])
         self._optimizer.load_state_dict(state_dict['optimizer'])
 
     def _init_collect(self) -> None:
@@ -291,9 +301,13 @@ class CollaQPolicy(Policy):
         Overview:
             Forward function for collect mode with eps_greedy
         Arguments:
-            - data (:obj:`dict`): Dict type data, including at least ['obs'].
+            - data (:obj:`Dict[str, Any]`): Dict type data, stacked env data for predicting policy_output(action), \
+                values are torch.Tensor or np.ndarray or dict/list combinations, keys are env_id indicated by integer.
+            - eps (:obj:`float`): epsilon value for exploration, which is decayed by collected env step.
         Returns:
-            - data (:obj:`dict`): The collected data
+            - output (:obj:`Dict[int, Any]`): Dict type data, including at least inferred action according to input obs.
+        ReturnsKeys
+            - necessary: ``action``
         """
         data_id = list(data.keys())
         data = default_collate(list(data.values()))
@@ -363,9 +377,12 @@ class CollaQPolicy(Policy):
         Overview:
             Forward function for eval mode, similar to ``self._forward_collect``.
         Arguments:
-            - data (:obj:`dict`): Dict type data, including at least ['obs'].
+            - data (:obj:`Dict[str, Any]`): Dict type data, stacked env data for predicting policy_output(action), \
+                values are torch.Tensor or np.ndarray or dict/list combinations, keys are env_id indicated by integer.
         Returns:
-            - output (:obj:`dict`): Dict type data, including at least inferred action according to input obs.
+            - output (:obj:`Dict[int, Any]`): The dict of predicting action for the interaction with env.
+        ReturnsKeys
+            - necessary: ``action``
         """
         data_id = list(data.keys())
         data = default_collate(list(data.values()))
