@@ -40,18 +40,32 @@ class DQNPolicy(Policy):
         8  | ``learn.update``   int      3              | How many updates(iterations) to train  | This args can be vary
            | ``per_collect``                            | after collector's one collection. Only | from envs. Bigger val
                                                         | valid in serial training               | means more off-policy
-        9  | ``learn.batch_``   int      64             | The number of samples of an iteration
+        9  | ``learn.multi``    bool     False          | whether to use multi gpu during
+           | ``_gpu``
+        10 | ``learn.batch_``   int      64             | The number of samples of an iteration
            | ``size``
-        10 | ``learn.learning`` float    0.001          | Gradient step length of an iteration.
+        11 | ``learn.learning`` float    0.001          | Gradient step length of an iteration.
            | ``_rate``
-        11 | ``learn.target_``  int      100            | Frequence of target network update.    | Hard(assign) update
+        12 | ``learn.target_``  int      100            | Frequence of target network update.    | Hard(assign) update
            | ``update_freq``
-        12 | ``learn.ignore_``  bool     False          | Whether ignore done for target value   | Enable it for some
+        13 | ``learn.ignore_``  bool     False          | Whether ignore done for target value   | Enable it for some
            | ``done``                                   | calculation.                           | fake termination env
-        13 ``collect.n_sample`` int      [8, 128]       | The number of training samples of a    | It varies from
+        14 ``collect.n_sample`` int      [8, 128]       | The number of training samples of a    | It varies from
                                                         | call of collector.                     | different envs
-        14 | ``collect.unroll`` int      1              | unroll length of an iteration          | In RNN, unroll_len>1
+        15 | ``collect.unroll`` int      1              | unroll length of an iteration          | In RNN, unroll_len>1
            | ``_len``
+        16 | ``other.eps.type`` str      exp            | exploration rate decay type            | Support ['exp',
+                                                                                                 | 'linear'].
+        17 | ``other.eps.       float    0.95           | start value of exploration rate        | [0,1]
+           |  start``
+        18 | ``other.eps.       float    0.1            | end value of exploration rate          | [0,1]
+           |  end``
+        19 | ``other.eps.       int      10000          | decay length of exploration            | greater than 0. set
+           |  decay``                                                                            | decay=10000 means
+                                                                                                 | the exploration rate
+                                                                                                 | decay from start
+                                                                                                 | value to end value
+                                                                                                 | during decay length.
         == ==================== ======== ============== ======================================== =======================
     """
 
@@ -191,11 +205,15 @@ class DQNPolicy(Policy):
         return {
             'cur_lr': self._optimizer.defaults['lr'],
             'total_loss': loss.item(),
+            'q_value': q_value.mean().item(),
             'priority': td_error_per_sample.abs().tolist(),
             # Only discrete action satisfying len(data['action'])==1 can return this and draw histogram on tensorboard.
             # '[histogram]action_distribution': data['action'],
         }
 
+    def _monitor_vars_learn(self) -> List[str]:
+        return ['cur_lr', 'total_loss', 'q_value']
+
     def _state_dict_learn(self) -> Dict[str, Any]:
         """
         Overview:
@@ -205,6 +223,7 @@ class DQNPolicy(Policy):
         """
         return {
             'model': self._learn_model.state_dict(),
+            'target_model': self._target_model.state_dict(),
             'optimizer': self._optimizer.state_dict(),
         }
 
@@ -221,6 +240,7 @@ class DQNPolicy(Policy):
             complicated operation.
         """
         self._learn_model.load_state_dict(state_dict['model'])
+        self._target_model.load_state_dict(state_dict['target_model'])
         self._optimizer.load_state_dict(state_dict['optimizer'])
 
     def _init_collect(self) -> None:
