@@ -14,6 +14,8 @@ PLAYER_REGISTRY = Registry()
 MODEL_REGISTRY = Registry()
 ENV_MANAGER_REGISTRY = Registry()
 REWARD_MODEL_REGISTRY = Registry()
+DATASET_REGISTRY = Registry()
+SERIAL_EVALUATOR_REGISTRY = Registry()
 
 registries = {
     'policy': POLICY_REGISTRY,
@@ -30,4 +32,6 @@ registries = {
     'league': LEAGUE_REGISTRY,
     'player': PLAYER_REGISTRY,
     'buffer': BUFFER_REGISTRY,
+    'dataset': DATASET_REGISTRY,
+    'serial_evaluator': SERIAL_EVALUATOR_REGISTRY,
 }
