@@ -1,9 +1,10 @@
+from time import time
 import pytest
 import numpy as np
 from dizoo.box2d.lunarlander.envs import LunarLanderEnv
 
 
-@pytest.mark.unittest
+@pytest.mark.envtest
 class TestLunarLanderEnvEnv:
 
     def test_naive(self):
