@@ -1,8 +1,10 @@
 from easydict import EasyDict
+import torch
 
 collector_env_num = 8
 evaluator_env_num = 5
 cartpole_r2d2_config = dict(
+    exp_name='cartpole_r2d2_bs20_n5_ul40_upc8_tuf2500_ed1e4_rbs1e5',
     env=dict(
         collector_env_num=collector_env_num,
         evaluator_env_num=evaluator_env_num,
@@ -11,34 +13,44 @@ cartpole_r2d2_config = dict(
     ),
     policy=dict(
         cuda=False,
-        on_policy=False,
         priority=False,
+        priority_IS_weight=False,
         model=dict(
             obs_shape=4,
             action_shape=2,
             encoder_hidden_size_list=[128, 128, 64],
         ),
         discount_factor=0.997,
-        burnin_step=5,
+        burnin_step=10,
         nstep=5,
+        # (int) the whole sequence length to unroll the RNN network minus
+        # the timesteps of burnin part,
+        # i.e., <the whole sequence length> = <burnin_step> + <unroll_len>
+        unroll_len=20,
         learn=dict(
-            update_per_collect=4,
+            # according to the R2D2 paper, actor parameter update interval is 400
+            # environment timesteps, and in per collect phase, we collect 32 sequence
+            # samples, the length of each samlpe sequence is <burnin_step> + <unroll_len>,
+            # which is 100 in our seeting, 32*100/400=8, so we set update_per_collect=8
+            # in most environments
+            update_per_collect=8,
             batch_size=64,
             learning_rate=0.0005,
-            target_update_freq=100,
+            # according to the R2D2 paper, the target network update interval is 2500
+            target_update_freq=2500,
         ),
         collect=dict(
             n_sample=32,
             env_num=collector_env_num,
         ),
-        eval=dict(env_num=evaluator_env_num, ),
+        eval=dict(env_num=evaluator_env_num, evaluator=dict(eval_freq=20)),
         other=dict(
             eps=dict(
                 type='exp',
                 start=0.95,
                 end=0.05,
-                decay=100000,
-            ), replay_buffer=dict(replay_buffer_size=5000, )
+                decay=10000,
+            ), replay_buffer=dict(replay_buffer_size=100000, )
         ),
     ),
 )
@@ -54,3 +66,7 @@ cartpole_r2d2_create_config = dict(
 )
 cartpole_r2d2_create_config = EasyDict(cartpole_r2d2_create_config)
 create_config = cartpole_r2d2_create_config
+
+if __name__ == "__main__":
+    from ding.entry import serial_pipeline
+    serial_pipeline([main_config, create_config], seed=0)
