@@ -5,7 +5,7 @@ from easydict import EasyDict
 from copy import deepcopy
 
 from ding.config import compile_config
-from ding.worker import BaseLearner, SampleCollector, BaseSerialEvaluator, AdvancedReplayBuffer
+from ding.worker import BaseLearner, SampleSerialCollector, InteractionSerialEvaluator, AdvancedReplayBuffer
 from ding.envs import BaseEnvManager, DingEnvWrapper
 from ding.policy import PPGPolicy
 from ding.model import PPG
@@ -23,8 +23,8 @@ def main(cfg, seed=0, max_iterations=int(1e10)):
         BaseEnvManager,
         PPGPolicy,
         BaseLearner,
-        SampleCollector,
-        BaseSerialEvaluator, {
+        SampleSerialCollector,
+        InteractionSerialEvaluator, {
             'policy': AdvancedReplayBuffer,
             'value': AdvancedReplayBuffer
         },
@@ -42,10 +42,10 @@ def main(cfg, seed=0, max_iterations=int(1e10)):
     policy = PPGPolicy(cfg.policy, model=model)
     tb_logger = SummaryWriter(os.path.join('./{}/log/'.format(cfg.exp_name), 'serial'))
     learner = BaseLearner(cfg.policy.learn.learner, policy.learn_mode, tb_logger, exp_name=cfg.exp_name)
-    collector = SampleCollector(
+    collector = SampleSerialCollector(
         cfg.policy.collect.collector, collector_env, policy.collect_mode, tb_logger, exp_name=cfg.exp_name
     )
-    evaluator = BaseSerialEvaluator(
+    evaluator = InteractionSerialEvaluator(
         cfg.policy.eval.evaluator, evaluator_env, policy.eval_mode, tb_logger, exp_name=cfg.exp_name
     )
     policy_buffer = AdvancedReplayBuffer(
@@ -66,7 +66,7 @@ def main(cfg, seed=0, max_iterations=int(1e10)):
         for i in range(cfg.policy.learn.update_per_collect):
             batch_size = learner.policy.get_attribute('batch_size')
             policy_data = policy_buffer.sample(batch_size['policy'], learner.train_iter)
-            value_data = policy_buffer.sample(batch_size['value'], learner.train_iter)
+            value_data = value_buffer.sample(batch_size['value'], learner.train_iter)
             if policy_data is not None and value_data is not None:
                 train_data = {'policy': policy_data, 'value': value_data}
                 learner.train(train_data, collector.envstep)
