@@ -4,7 +4,7 @@ import numpy as np
 from dizoo.minigrid.envs import MiniGridEnv
 
 
-@pytest.mark.unittest
+@pytest.mark.envtest
 class TestMiniGridEnv:
 
     def test_naive(self):
@@ -18,10 +18,11 @@ class TestMiniGridEnv:
         obs = env.reset()
         act_val = env.info().act_space.value
         min_val, max_val = act_val['min'], act_val['max']
-        for i in range(100):
+        for i in range(env._max_step):
             random_action = np.random.randint(min_val, max_val, size=(1, ))
             timestep = env.step(random_action)
             print(timestep)
+            print(timestep.obs.max())
             assert isinstance(timestep.obs, np.ndarray)
             assert isinstance(timestep.done, bool)
             assert timestep.obs.shape == (2739, )
